unit XDBGridUnit1;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  Db, DBTables, Grids, XDBGrids, StdCtrls, ExtCtrls, CheckLst, Spin;

type
  TXDBGridForm1 = class(TForm)
    ColorDialog: TColorDialog;
    FontDialog: TFontDialog;
    CustomerDataSource: TDataSource;
      Customer: TQuery;
    CustomerPanel: TPanel;
      CustomerDBGrid: TXDBGrid;
    CommentsPanel: TPanel;
      HeadersPanel: TPanel;
        DBGridOptionsLabel: TLabel;
        XDBGridOptionsLabel: TLabel;
        TitlePropertyLabel: TLabel;
        ColumnPropertyLabel: TLabel;
        ColorPropertyLabel: TLabel;
        FontPropertyLabel: TLabel;
      OptionsPanel: TPanel;
        OptionsCheckListBox: TCheckListBox;
        TitleCaptionEdit: TEdit;
        TitleAlignmentComboBox: TComboBox;
        TitleVAlignmentComboBox: TComboBox;
        TitleButtonCheckBox: TCheckBox;
        TitleEllipsisCheckBox: TCheckBox;
        TitleWordWrapCheckBox: TCheckBox;
        ColumnTextEdit: TEdit;
        ColumnAlignmentComboBox: TComboBox;
        ColumnVAlignmentComboBox: TComboBox;
        ColumnShowEditCheckBox: TCheckBox;
        ColumnEllipsisCheckBox: TCheckBox;
        ColumnWordWrapCheckBox: TCheckBox;
        HeaderClickPanel: TPanel;
        GridColorButton: TButton;
        HeaderColorButton: TButton;
        TitleColorButton: TButton;
        ColumnColorButton: TButton;
        GridFontButton: TButton;
        HeaderFontButton: TButton;
        TitleFontButton: TButton;
        ColumnFontButton: TButton;
        LinesCountPropertyLabel: TLabel;
        LinesCountPropertyBevel: TBevel;
        HeaderLabel: TLabel;
        TitleLabel: TLabel;
        ColumnLabel: TLabel;
        HeaderSpinEdit: TSpinEdit;
        TitleSpinEdit: TSpinEdit;
        ColumnSpinEdit: TSpinEdit;
    procedure FormCreate(Sender: TObject);
    procedure OptionsCheckListBoxClickCheck(Sender: TObject);
    procedure ControlsChange(Sender: TObject);
    procedure CustomerDBGridColEnter(Sender: TObject);
    procedure CustomerAfterScroll(DataSet: TDataSet);
    procedure CustomerDBGridTitleClick(Column: TXColumn);
    procedure CustomerDBGridHeaderClick(Column: TXColumn; Index: Integer);
    procedure GridColorButtonClick(Sender: TObject);
    procedure HeaderColorButtonClick(Sender: TObject);
    procedure TitleColorButtonClick(Sender: TObject);
    procedure ColumnColorButtonClick(Sender: TObject);
    procedure GridFontButtonClick(Sender: TObject);
    procedure HeaderFontButtonClick(Sender: TObject);
    procedure TitleFontButtonClick(Sender: TObject);
    procedure ColumnFontButtonClick(Sender: TObject);
  private
    FRestore: Boolean;
    procedure RestoreOptions;
    procedure SetupOptions;
    procedure RestoreProperty;
    procedure SetupProperty;
  end;

var
  XDBGridForm1: TXDBGridForm1;

implementation

{$R *.DFM}

procedure TXDBGridForm1.RestoreOptions;
var
  Option: TXDBGridOption;
begin
  with CustomerDBGrid do
  for Option := Low(TXDBGridOption) to High(TXDBGridOption) do
    OptionsCheckListBox.Checked[Ord(Option)] := Option in Options;
end;

procedure TXDBGridForm1.SetupOptions;
var
  Option: TXDBGridOption;
begin
  with CustomerDBGrid do
  for Option := Low(TXDBGridOption) to High(TXDBGridOption) do
    if OptionsCheckListBox.Checked[Ord(Option)] then Options := Options + [Option]
    else Options := Options - [Option];
end;

procedure TXDBGridForm1.RestoreProperty;
var
  Column: TXColumn;
begin
  FRestore := True;
  with CustomerDBGrid do Column := Columns[SelectedIndex];
  TitleCaptionEdit.Text := Column.Title.Caption;
  TitleAlignmentComboBox.ItemIndex := Ord(Column.Title.Alignment);
  TitleVAlignmentComboBox.ItemIndex := Ord(Column.Title.VAlignment);
  TitleButtonCheckBox.Checked := Column.Title.Button;
  TitleEllipsisCheckBox.Checked := Column.Title.Ellipsis;
  TitleWordWrapCheckBox.Checked := Column.Title.WordWrap;
  ColumnTextEdit.Text := Column.Field.Text;
  ColumnAlignmentComboBox.ItemIndex := Ord(Column.Alignment);
  ColumnVAlignmentComboBox.ItemIndex := Ord(Column.VAlignment);
  ColumnShowEditCheckBox.Checked := Column.ShowEdit;
  ColumnEllipsisCheckBox.Checked := Column.Ellipsis;
  ColumnWordWrapCheckBox.Checked := Column.WordWrap;
  HeaderSpinEdit.Value := CustomerDBGrid.HeaderLinesCount;
  TitleSpinEdit.Value := CustomerDBGrid.TitleLinesCount;
  ColumnSpinEdit.Value := CustomerDBGrid.LinesCount;
  HeaderClickPanel.Caption := '';
  FRestore := False;
end;

procedure TXDBGridForm1.SetupProperty;
var
  Column: TXColumn;
begin
  if FRestore then Exit;
  with CustomerDBGrid do Column := Columns[SelectedIndex];
  Column.Title.Caption := TitleCaptionEdit.Text;
  Column.Title.Alignment := TAlignment(TitleAlignmentComboBox.ItemIndex);
  Column.Title.VAlignment := TVAlignment(TitleVAlignmentComboBox.ItemIndex);
  Column.Title.Button := TitleButtonCheckBox.Checked;
  Column.Title.Ellipsis := TitleEllipsisCheckBox.Checked;
  Column.Title.WordWrap := TitleWordWrapCheckBox.Checked;
  Column.Alignment := TAlignment(ColumnAlignmentComboBox.ItemIndex);
  Column.VAlignment := TVAlignment(ColumnVAlignmentComboBox.ItemIndex);
  Column.ShowEdit := ColumnShowEditCheckBox.Checked;
  Column.Ellipsis := ColumnEllipsisCheckBox.Checked;
  Column.WordWrap := ColumnWordWrapCheckBox.Checked;
  CustomerDBGrid.HeaderLinesCount := HeaderSpinEdit.Value;
  CustomerDBGrid.TitleLinesCount := TitleSpinEdit.Value;
  CustomerDBGrid.LinesCount := ColumnSpinEdit.Value;
end;

procedure TXDBGridForm1.FormCreate(Sender: TObject);
begin
  RestoreOptions;
  RestoreProperty;
end;

procedure TXDBGridForm1.OptionsCheckListBoxClickCheck(Sender: TObject);
begin
  SetupOptions;
  RestoreOptions;
end;

procedure TXDBGridForm1.ControlsChange(Sender: TObject);
begin
  SetupProperty;
end;

procedure TXDBGridForm1.CustomerDBGridColEnter(Sender: TObject);
begin
  RestoreProperty;
end;

procedure TXDBGridForm1.CustomerAfterScroll(DataSet: TDataSet);
begin
  RestoreProperty;
end;

procedure TXDBGridForm1.CustomerDBGridTitleClick(Column: TXColumn);
const
  OrderBy: array[TMarker] of string = ('', 'ASC', 'DESC');
  SQLText = 'SELECT * FROM Customer ORDER BY %S %S';
  CustNo = 'CustNo';
var
  Current: string;
begin
  with Column.Title do
  begin
    if not Button then Exit;
    if Marker <> tmAscend then Marker := tmAscend else Marker := tmDescend;
    Customer.DisableControls;
    try
      Current := Customer.FieldByName(CustNo).AsString;
      Customer.Close;
      Customer.SQL.Clear;
      Customer.SQL.Add(Format(SQLText, [Column.FieldName, OrderBy[Marker]]));
      Customer.Open;
      Customer.Locate(CustNo, Current, []);
    finally
      Customer.EnableControls;
    end;
  end;
end;

procedure TXDBGridForm1.CustomerDBGridHeaderClick(Column: TXColumn; Index: Integer);
begin
  with Column.Title do
  HeaderClickPanel.Caption := HeaderRows[Index] + '->' + Caption + ' clicked.';
end;

procedure TXDBGridForm1.GridColorButtonClick(Sender: TObject);
begin
  ColorDialog.Color := CustomerDBGrid.Color;
  if ColorDialog.Execute then CustomerDBGrid.Color := ColorDialog.Color;
end;

procedure TXDBGridForm1.HeaderColorButtonClick(Sender: TObject);
begin
  ColorDialog.Color := CustomerDBGrid.HeaderColor;
  if ColorDialog.Execute then CustomerDBGrid.HeaderColor := ColorDialog.Color;
end;

procedure TXDBGridForm1.TitleColorButtonClick(Sender: TObject);
var
  Column: TXColumn;
begin
  with CustomerDBGrid do Column := Columns[SelectedIndex];
  ColorDialog.Color := Column.Title.Color;
  if ColorDialog.Execute then Column.Title.Color := ColorDialog.Color;
end;

procedure TXDBGridForm1.ColumnColorButtonClick(Sender: TObject);
var
  Column: TXColumn;
begin
  with CustomerDBGrid do Column := Columns[SelectedIndex];
  ColorDialog.Color := Column.Color;
  if ColorDialog.Execute then Column.Color := ColorDialog.Color;
end;

procedure TXDBGridForm1.GridFontButtonClick(Sender: TObject);
begin
  FontDialog.Font := TPanel(CustomerDBGrid.Parent).Font;
  if FontDialog.Execute then
  begin
    TPanel(CustomerDBGrid.Parent).Font := FontDialog.Font;
    CustomerDBGrid.ParentFont := True;
  end;
end;

procedure TXDBGridForm1.HeaderFontButtonClick(Sender: TObject);
begin
  FontDialog.Font := CustomerDBGrid.HeaderFont;
  if FontDialog.Execute then CustomerDBGrid.HeaderFont := FontDialog.Font;
end;

procedure TXDBGridForm1.TitleFontButtonClick(Sender: TObject);
var
  Column: TXColumn;
begin
  with CustomerDBGrid do Column := Columns[SelectedIndex];
  FontDialog.Font := Column.Title.Font;
  if FontDialog.Execute then Column.Title.Font := FontDialog.Font;
end;

procedure TXDBGridForm1.ColumnFontButtonClick(Sender: TObject);
var
  Column: TXColumn;
begin
  with CustomerDBGrid do Column := Columns[SelectedIndex];
  FontDialog.Font := Column.Font;
  if FontDialog.Execute then Column.Font := FontDialog.Font;
end;

end.
